@@ -661,9 +661,11 @@ class AvatarServiceRegistryTests(SpyAgency, TestCase):
 
         siteconfig = SiteConfiguration.objects.get_current()
         self.assertEqual(
-            siteconfig.get(AvatarServiceRegistry.ENABLED_SERVICES_KEY),
-            [DummyAvatarService.avatar_service_id,
-             GravatarService.avatar_service_id])
+            set(siteconfig.get(AvatarServiceRegistry.ENABLED_SERVICES_KEY)),
+            {
+                DummyAvatarService.avatar_service_id,
+                GravatarService.avatar_service_id,
+            })
 
     def test_set_enabled_services_with_save_false(self):
         """Testing AvatarServiceRegistry.set_enabled_services with save=False
@@ -701,7 +703,7 @@ class AvatarServiceRegistryTests(SpyAgency, TestCase):
                     return
 
                 super(TestRegistry, self).populate()
-                self.enabled_services = [DummyAvatarService]
+                self.set_enabled_services([DummyAvatarService])
 
             def get_defaults(self):
                 yield DummyAvatarService
