@@ -27,7 +27,6 @@ from __future__ import unicode_literals
 
 import warnings
 
-from djblets.deprecation import RemovedInDjblets20Warning
 from djblets.forms.forms import KeyValueForm
 from djblets.util.decorators import cached_property
 
@@ -48,16 +47,6 @@ class SettingsForm(KeyValueForm):
         super(SettingsForm, self).__init__(instance=extension.settings,
                                            *args, **kwargs)
 
-    @cached_property
-    def siteconfig(self):
-        """An alias for the instance, for backwards-compatibility."""
-        warnings.warn(
-            'SettingsForm.siteconfig is deprecated. Update your code to '
-            'use SettingsForm.settings instead.',
-            RemovedInDjblets20Warning)
-
-        return self.instance
-
     def set_key_value(self, key, value):
         """Set the value for an extension settings key.
 
