@@ -10,6 +10,7 @@ import sys
 from distutils.errors import DistutilsExecError
 from fnmatch import fnmatch
 
+import django
 import pkg_resources
 from django.core.management import call_command
 from django.utils import six
@@ -140,10 +141,10 @@ class BuildStaticFiles(Command):
         node_modules_dir = os.path.join(build_dir, 'node_modules')
 
         if not os.path.exists(build_dir):
-            os.mkdir(build_dir, 0755)
+            os.mkdir(build_dir, 0o755)
 
         if not os.path.exists(node_modules_dir):
-            os.mkdir(node_modules_dir, 0755)
+            os.mkdir(node_modules_dir, 0o755)
 
         dependencies = {}
 
@@ -237,7 +238,7 @@ class BuildStaticFiles(Command):
             # Ensure there's a node_modules directory here, so it doesn't
             # install higher up in the directory hierarchy.
             if not os.path.exists('node_modules'):
-                os.mkdir('node_modules', 0755)
+                os.mkdir('node_modules', 0o755)
 
             print('Installing %s...' % package_spec)
             result = os.system('npm install %s' % package_spec)
@@ -256,6 +257,11 @@ class BuildStaticFiles(Command):
         # media.
         old_settings_module = os.environ.get('DJANGO_SETTINGS_MODULE')
         os.environ['DJANGO_SETTINGS_MODULE'] = self.django_settings_module
+
+        if hasattr(django, 'setup'):
+            # Django >= 1.7
+            django.setup()
+
         cwd = os.getcwd()
         sys.path = [
             os.path.join(cwd, package_name)
