@@ -27,15 +27,16 @@ from __future__ import unicode_literals
 
 from django.contrib.admin.views.decorators import staff_member_required
 from django.http import Http404, HttpResponseRedirect
-from django.shortcuts import render
 from django.template.context import RequestContext
 from django.views.decorators.csrf import csrf_protect
 
+from djblets.util.compat.django.shortcuts import render
+
 
 @csrf_protect
 @staff_member_required
 def extension_list(request, extension_manager,
-                   template_name='extensions/extension_list.html'):
+                   template_name='extensions/admin/extension_list.html'):
     if request.method == 'POST':
         if 'full-reload' in request.POST:
             extension_manager.load(full_reload=True)
