@@ -26,18 +26,35 @@
 
 from __future__ import print_function, unicode_literals
 
-import imp
+import importlib
 import inspect
 import os
 import re
 import socket
 import sys
 import threading
+import types
+import warnings
 from collections import OrderedDict
 from contextlib import contextmanager
 from importlib import import_module
 from unittest.util import safe_repr
 
+try:
+    # Python >= 3.4
+    from importlib.machinery import ModuleSpec
+
+    try:
+        # Python >= 3.5
+        from importlib.util import module_from_spec
+    except ImportError:
+        # Python < 3.5
+        module_from_spec = None
+except ImportError:
+    # Python < 3.4
+    ModuleSpec = None
+    module_from_spec = None
+
 from django.conf import settings
 from django.core import serializers
 from django.core.exceptions import ValidationError
@@ -50,7 +67,6 @@ from django.db.models import Model
 from django.template import Node
 from django.test import testcases
 from django.utils import six
-from django.utils.module_loading import module_has_submodule
 
 try:
     # Django >= 1.7
@@ -61,6 +77,12 @@ except ImportError:
 
     apps = None
 
+try:
+    from django_evolution.models import Evolution, Version
+except ImportError:
+    Evolution = None
+    Version = None
+
 from djblets.siteconfig.models import SiteConfiguration
 
 
@@ -237,6 +259,55 @@ class TestCase(testcases.TestCase):
             self, expected_exception, re.escape(expected_message),
             *args, **kwargs)
 
+    @contextmanager
+    def assertWarns(self, cls=DeprecationWarning, message=None):
+        """Assert that a warning is generated with a given message.
+
+        This method only supports code which generates a single warning.
+        Tests which make use of code generating multiple warnings will
+        need to manually catch their warnings.
+
+        Args:
+            cls (type, optional):
+                The expected warning type.
+
+            message (unicode, optional):
+                The expected error message, if any.
+
+        Context:
+            The test to run.
+        """
+        with warnings.catch_warnings(record=True) as w:
+            # Some warnings such as DeprecationWarning are filtered by
+            # default, stop filtering them.
+            warnings.simplefilter('always')
+            self.assertEqual(len(w), 0)
+
+            yield
+
+            self.assertEqual(len(w), 1)
+            self.assertTrue(issubclass(w[-1].category, cls))
+
+            if message is not None:
+                self.assertEqual(message, six.text_type(w[-1].message))
+
+    @contextmanager
+    def assertNoWarnings(self):
+        """Assert that a warning is not generated.
+
+        Context:
+            The test to run.
+        """
+        with warnings.catch_warnings(record=True) as w:
+            # Some warnings such as DeprecationWarning are filtered by
+            # default, stop filtering them.
+            warnings.simplefilter('always')
+            self.assertEqual(len(w), 0)
+
+            yield
+
+            self.assertEqual(len(w), 0)
+
 
 class TestModelsLoaderMixin(object):
     """Allows unit test modules to provide models to test against.
@@ -262,8 +333,6 @@ class TestModelsLoaderMixin(object):
 
     @classmethod
     def setUpClass(cls):
-        super(TestModelsLoaderMixin, cls).setUpClass()
-
         cls._tests_loader_models_mod = None
 
         if not cls.tests_app:
@@ -276,14 +345,37 @@ class TestModelsLoaderMixin(object):
         except ImportError:
             # Set up a 'models' module, containing any models local to the
             # module that this TestCase is in.
-            models_mod = imp.new_module(models_mod_name)
-            module_name = cls.__module__
+            if ModuleSpec:
+                # Python >= 3.4
+                #
+                # It's not enough to simply create a module type. We need to
+                # create a basic spec, and then we need to have the module
+                # system create a module from it. There's a handy public
+                # function to do this on Python 3.5, but Python 3.4 lacks a
+                # public function. Fortunately, it's easy to call a private
+                # one.
+                spec = ModuleSpec(name=models_mod_name,
+                                  loader=None)
 
-            # Django needs a value here. Doesn't matter what it is.
-            models_mod.__file__ = ''
+                if module_from_spec:
+                    # Python >= 3.5
+                    models_mod = module_from_spec(spec)
+                else:
+                    # Python == 3.4
+                    models_mod = \
+                        importlib._bootstrap._SpecMethods(spec).create()
+
+                assert models_mod
+            else:
+                # Python < 3.4
+                models_mod = types.ModuleType(str(models_mod_name))
+
+                # Django needs a value here. Doesn't matter what it is.
+                models_mod.__file__ = ''
 
             # Transfer all the models over into this new module.
-            test_module = sys.modules[cls.__module__]
+            module_name = cls.__module__
+            test_module = sys.modules[module_name]
 
             for key, value in six.iteritems(test_module.__dict__):
                 if (inspect.isclass(value) and
@@ -293,32 +385,28 @@ class TestModelsLoaderMixin(object):
 
         cls._tests_loader_models_mod = models_mod
 
-    @classmethod
-    def tearDownClass(cls):
-        super(TestModelsLoaderMixin, cls).tearDownClass()
-
-        # Set this free so the garbage collector can eat it.
-        cls._tests_loader_models_mod = None
-
-    def setUp(self):
-        super(TestModelsLoaderMixin, self).setUp()
-
-        # If we made a fake 'models' module, add it to sys.modules.
-        models_mod = self._tests_loader_models_mod
-
         if models_mod:
             sys.modules[models_mod.__name__] = models_mod
 
-        self._models_loader_old_settings = settings.INSTALLED_APPS
+        cls._models_loader_old_settings = settings.INSTALLED_APPS
         settings.INSTALLED_APPS = list(settings.INSTALLED_APPS) + [
-            self.tests_app,
+            cls.tests_app,
         ]
 
+        # If Django Evolution is being used, we'll want to clear out any
+        # recorded schema information so that it can be generated from
+        # scratch when we set up the database.
+        if (Evolution is not None and
+            Version is not None and
+            getattr(settings, 'DJANGO_EVOLUTION_ENABLED', True)):
+            Version.objects.all().delete()
+            Evolution.objects.all().delete()
+
         if apps:
             # Push the new set of installed apps, and begin registering
             # each of the models associated with the tests.
             apps.set_installed_apps(settings.INSTALLED_APPS)
-            app_config = apps.get_containing_app_config(self.tests_app)
+            app_config = apps.get_containing_app_config(cls.tests_app)
 
             if models_mod:
                 app_label = app_config.label
@@ -340,18 +428,21 @@ class TestModelsLoaderMixin(object):
             call_command('migrate', run_syncdb=True, verbosity=0,
                          interactive=False)
         else:
-            load_app(self.tests_app)
+            load_app(cls.tests_app)
             call_command('syncdb', verbosity=0, interactive=False)
 
-    def tearDown(self):
-        super(TestModelsLoaderMixin, self).tearDown()
+        super(TestModelsLoaderMixin, cls).setUpClass()
+
+    @classmethod
+    def tearDownClass(cls):
+        super(TestModelsLoaderMixin, cls).tearDownClass()
 
         call_command('flush', verbosity=0, interactive=False)
 
-        settings.INSTALLED_APPS = self._models_loader_old_settings
+        settings.INSTALLED_APPS = cls._models_loader_old_settings
 
         # If we added a fake 'models' module to sys.modules, remove it.
-        models_mod = self._tests_loader_models_mod
+        models_mod = cls._tests_loader_models_mod
 
         if models_mod:
             try:
@@ -361,13 +452,16 @@ class TestModelsLoaderMixin(object):
 
         if apps:
             apps.unset_installed_apps()
-            apps.all_models[self.tests_app].clear()
+            apps.all_models[cls.tests_app].clear()
         else:
             if models_mod:
                 del app_cache.app_store[models_mod]
 
             app_cache._get_models_cache.clear()
 
+        # Set this free so the garbage collector can eat it.
+        cls._tests_loader_models_mod = None
+
 
 class FixturesCompilerMixin(object):
     """Compiles and efficiently loads fixtures into a test suite.
