@@ -1,17 +1,62 @@
 from __future__ import print_function, unicode_literals
 
+import json
 import warnings
 
 from django.contrib.auth.models import User
 from django.db.models import Model
+from django.http import HttpResponseNotModified
 from django.test.client import RequestFactory
 from django.utils import six
 
 from djblets.testing.testcases import TestCase
+from djblets.util.http import encode_etag
+from djblets.webapi.fields import (ResourceFieldType,
+                                   ResourceListFieldType,
+                                   StringFieldType)
 from djblets.webapi.resources.base import WebAPIResource
 from djblets.webapi.resources.registry import (register_resource_for_model,
                                                unregister_resource_for_model,
                                                unregister_resource)
+from djblets.webapi.responses import WebAPIResponse
+
+
+class BaseTestWebAPIResource(WebAPIResource):
+    mimetype_vendor = 'djblets-test'
+
+
+class TestUserResource(BaseTestWebAPIResource):
+    name = 'user'
+    model = User
+
+    fields = {
+        'username': {
+            'type': StringFieldType,
+        },
+        'first_name': {
+            'type': StringFieldType,
+        },
+        'last_name': {
+            'type': StringFieldType,
+        },
+    }
+
+    def get_href(self, obj, *args, **kwargs):
+        return 'http://testserver/api/test/users/%s/' % obj.pk
+
+
+class BaseTestRefUserResource(BaseTestWebAPIResource):
+    name = 'test'
+    uri_object_key = 'pk'
+
+    def build_resource_url(self, name, request, **kwargs):
+        return request.build_absolute_uri()
+
+    def get_serializer_for_object(self, o):
+        if isinstance(o, User):
+            return TestUserResource()
+        else:
+            return self
 
 
 class WebAPIResourceTests(TestCase):
@@ -20,6 +65,8 @@ class WebAPIResourceTests(TestCase):
     def setUp(self):
         super(WebAPIResourceTests, self).setUp()
 
+        self.maxDiff = 10000000
+
         self.factory = RequestFactory()
         self.test_resource = None
 
@@ -215,6 +262,59 @@ class WebAPIResourceTests(TestCase):
             view_kwargs={'id': 1},
             method='delete')
 
+    def test_get_with_autogenerate_etags_and_not_modified(self):
+        """Testing WebAPIResource with GET and autogenerate_etags and requested
+        ETag is a match
+        """
+        class TestResource(WebAPIResource):
+            allowed_methods = ('GET', 'POST', 'PUT', 'DELETE')
+            mimetype_vendor = 'djblets'
+            uri_object_key = 'id'
+            autogenerate_etags = True
+
+            def get(self, *args, **kwargs):
+                return 200, {
+                    'abc': True,
+                }
+
+        resource = TestResource()
+        etag = '790fc95c3f89afe28403c272553e790274ea1d3e'
+
+        request = self.factory.get('/api/tests/',
+                                   HTTP_ACCEPT='application/json',
+                                   HTTP_IF_NONE_MATCH=etag)
+        response = resource(request)
+
+        self.assertIsInstance(response, HttpResponseNotModified)
+        self.assertEqual(response.status_code, 304)
+        self.assertEqual(response['ETag'], etag)
+
+    def test_get_with_autogenerate_etags_and_modified(self):
+        """Testing WebAPIResource with GET and autogenerate_etags and
+        requested ETag is not a match
+        """
+        class TestResource(WebAPIResource):
+            allowed_methods = ('GET', 'POST', 'PUT', 'DELETE')
+            mimetype_vendor = 'djblets'
+            uri_object_key = 'id'
+            autogenerate_etags = True
+
+            def get(self, *args, **kwargs):
+                return 200, {
+                    'abc': True,
+                }
+
+        resource = TestResource()
+        request = self.factory.get('/api/tests/',
+                                   HTTP_ACCEPT='application/json',
+                                   HTTP_IF_NONE_MATCH='adfjhslfa')
+        response = resource(request)
+
+        self.assertIsInstance(response, WebAPIResponse)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(response['ETag'],
+                         '790fc95c3f89afe28403c272553e790274ea1d3e')
+
     def test_generate_etag_with_encode_etag_true(self):
         """Testing WebAPIResource.generate_etag with encode_etag=True"""
         class TestObject(object):
@@ -224,15 +324,21 @@ class WebAPIResourceTests(TestCase):
         request.user = User()
 
         resource = WebAPIResource()
+        obj = TestObject()
 
         with warnings.catch_warnings(record=True) as w:
-            etag = resource.generate_etag(TestObject(), ['my_field'], request,
+            warnings.simplefilter('always')
+            etag = resource.generate_etag(obj, ['my_field'], request,
                                           encode_etag=True)
-            self.assertEqual(len(w), 1)
-            self.assertIn('generate_etag will stop generating',
-                          six.text_type(w[0].message))
 
-        self.assertEqual(etag, '416c0aecaf0b1e8ec64104349ba549c7534861f2')
+        self.assertEqual(len(w), 1)
+        self.assertIn('generate_etag is deprecated',
+                      six.text_type(w[0].message))
+
+        self.assertEqual(
+            etag,
+            encode_etag(
+                ':%s' % repr(resource.serialize_object(obj, request=request))))
 
     def test_generate_etag_with_encode_etag_false(self):
         """Testing WebAPIResource.generate_etag with encode_etag=False"""
@@ -246,9 +352,13 @@ class WebAPIResourceTests(TestCase):
         obj = TestObject()
 
         with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
             etag = resource.generate_etag(obj, None, request,
                                           encode_etag=False)
-            self.assertEqual(len(w), 0)
+
+        self.assertEqual(len(w), 1)
+        self.assertIn('generate_etag is deprecated',
+                      six.text_type(w[0].message))
 
         self.assertEqual(
             etag,
@@ -310,13 +420,13 @@ class WebAPIResourceTests(TestCase):
         class TestResource(WebAPIResource):
             fields = {
                 'field1': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
                 'field2': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
                 'field3': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
             }
 
@@ -346,13 +456,13 @@ class WebAPIResourceTests(TestCase):
         class TestResource(WebAPIResource):
             fields = {
                 'field1': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
                 'field2': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
                 'field3': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
             }
 
@@ -369,87 +479,248 @@ class WebAPIResourceTests(TestCase):
             }
         })
 
-    def test_serialize_object_with_only_fields_and_expand(self):
+    def test_serialize_object_with_expand_model(self):
         """Testing WebAPIResource.serialize_object with
-        ?only-fields=<field>&expand=<field>
+        ?expand=<model_field>
         """
-        class TestModel(Model):
-            field = 'test'
-
-            def __init__(self, pk):
-                self.pk = pk  # Django 1.8+ requires a pk field
-
-            # TestModel isn't a real model picked up by Django, and this
-            # causes deepcopy to fail with it. We need to override for
-            # the sake of unit tests.
-            def __deepcopy__(self, other):
-                return TestModel()
-
-            def __str__(self):
-                return 'Test'
-
         class TestObject(object):
-            field1 = 'abc'
-            field2 = TestModel(1)
-            field3 = TestModel(2)
+            pk = '1'
+            user = User.objects.create(username='user1',
+                                       first_name='Test1',
+                                       last_name='User1')
 
-        class TestResource1(WebAPIResource):
+        class TestResource(BaseTestRefUserResource):
             fields = {
-                'field': {
-                    'type': six.text_type,
+                'user': {
+                    'type': ResourceFieldType,
+                    'resource': TestUserResource,
                 },
             }
 
-            def get_href(self, *args, **kwargs):
-                return 'http://testserver/api/test1/'
+        request = RequestFactory().get('/api/test/?expand=user')
+        resource = TestResource()
+        data = resource.serialize_object(TestObject(), request=request)
 
-        class TestResource2(WebAPIResource):
-            fields = {
-                'field1': {
-                    'type': six.text_type,
-                },
-                'field2': {
-                    'type': TestModel,
-                },
-                'field3': {
-                    'type': TestModel,
-                },
-            }
-
-            def get_serializer_for_object(self, o):
-                if isinstance(o, TestModel):
-                    return TestResource1()
-                else:
-                    return self
-
-        request = RequestFactory().get(
-            '/api/test2/?only-fields=field2&expand=field2')
-        resource = TestResource2()
-        obj = TestObject()
-        data = resource.serialize_object(obj, request=request)
-
-        # Note that field2 below isn't a serialized version of a TestModel.
-        # That's because serialization of model instances happens when
-        # dumping to a JSON string. We're not testing that part.
         self.assertEqual(data, {
-            'field2': {
+            '_expanded': {
+                'user': {
+                    'item_mimetype': 'application/vnd.djblets-test.user+json',
+                },
+            },
+            'user': {
+                'username': 'user1',
+                'first_name': 'Test1',
+                'last_name': 'User1',
                 'links': {
                     'self': {
-                        'href': 'http://testserver/api/test1/',
+                        'href': 'http://testserver/api/test/users/1/',
                         'method': 'GET',
                     },
                 },
             },
             'links': {
                 'self': {
-                    'href': 'http://testserver/api/test2/?only-fields=field2'
+                    'href': 'http://testserver/api/test/?expand=user',
+                    'method': 'GET',
+                },
+            }
+        })
+
+    def test_serialize_object_with_expand_queryset(self):
+        """Testing WebAPIResource.serialize_object with
+        ?expand=<queryset_field>
+        """
+        class TestObject(object):
+            pk = '1'
+            users = User.objects.all()
+
+        class TestResource(BaseTestRefUserResource):
+            fields = {
+                'users': {
+                    'type': ResourceListFieldType,
+                    'resource': TestUserResource,
+                },
+            }
+
+        User.objects.create(username='user1',
+                            first_name='Test1',
+                            last_name='User1')
+        User.objects.create(username='user2',
+                            first_name='Test2',
+                            last_name='User2')
+
+        request = RequestFactory().get('/api/test/?expand=users')
+        resource = TestResource()
+        data = resource.serialize_object(TestObject(), request=request)
+
+        self.assertEqual(data, {
+            '_expanded': {
+                'users': {
+                    'item_mimetype': 'application/vnd.djblets-test.user+json',
+                },
+            },
+            'users': [
+                {
+                    'username': 'user1',
+                    'first_name': 'Test1',
+                    'last_name': 'User1',
+                    'links': {
+                        'self': {
+                            'href': 'http://testserver/api/test/users/1/',
+                            'method': 'GET',
+                        },
+                    },
+                },
+                {
+                    'username': 'user2',
+                    'first_name': 'Test2',
+                    'last_name': 'User2',
+                    'links': {
+                        'self': {
+                            'href': 'http://testserver/api/test/users/2/',
+                            'method': 'GET',
+                        },
+                    },
+                },
+            ],
+            'links': {
+                'self': {
+                    'href': 'http://testserver/api/test/?expand=users',
+                    'method': 'GET',
+                },
+            }
+        })
+
+    def test_serialize_object_with_expand_child_resource(self):
+        """Testing WebAPIResource.serialize_object with
+        ?expand=<child_resource_field>
+        """
+        class TestObject(object):
+            pk = '1'
+            field = 'test'
+            users = User.objects.all()
+
+        class TestResource(BaseTestRefUserResource):
+            item_child_resources = [TestUserResource()]
+
+            fields = {
+                'field': {
+                    'type': StringFieldType,
+                },
+            }
+
+        User.objects.create(username='user1',
+                            first_name='Test1',
+                            last_name='User1')
+        User.objects.create(username='user2',
+                            first_name='Test2',
+                            last_name='User2')
+
+        request = RequestFactory().get('/api/test/?expand=users')
+        resource = TestResource()
+        data = resource.serialize_object(TestObject(), request=request)
+
+        self.assertEqual(data, {
+            '_expanded': {
+                'users': {
+                    'item_mimetype': 'application/vnd.djblets-test.user+json',
+                    'list_mimetype': 'application/vnd.djblets-test.users+json',
+                    'list_url': 'http://testserver/api/test/users/',
+                },
+            },
+            'field': 'test',
+            'users': [
+                {
+                    'username': 'user1',
+                    'first_name': 'Test1',
+                    'last_name': 'User1',
+                    'links': {
+                        'self': {
+                            'href': 'http://testserver/api/test/users/1/',
+                            'method': 'GET',
+                        },
+                    },
+                },
+                {
+                    'username': 'user2',
+                    'first_name': 'Test2',
+                    'last_name': 'User2',
+                    'links': {
+                        'self': {
+                            'href': 'http://testserver/api/test/users/2/',
+                            'method': 'GET',
+                        },
+                    },
+                },
+            ],
+            'links': {
+                'self': {
+                    'href': 'http://testserver/api/test/?expand=users',
+                    'method': 'GET',
+                },
+            }
+        })
+
+    def test_serialize_object_with_only_fields_and_expand(self):
+        """Testing WebAPIResource.serialize_object with
+        ?only-fields=<field>&expand=<field>
+        """
+        class TestObject(object):
+            pk = 1
+
+            field1 = 'abc'
+            field2 = User.objects.create(username='user1',
+                                         first_name='Test1',
+                                         last_name='Test1')
+            field3 = User.objects.create(username='user2',
+                                         first_name='Test2',
+                                         last_name='Test2')
+
+        class TestResource(BaseTestRefUserResource):
+            fields = {
+                'field1': {
+                    'type': StringFieldType,
+                },
+                'field2': {
+                    'type': ResourceFieldType,
+                    'resource': TestUserResource,
+                },
+                'field3': {
+                    'type': ResourceFieldType,
+                    'resource': TestUserResource,
+                },
+            }
+
+        request = RequestFactory().get(
+            '/api/test/?only-fields=field2&expand=field2')
+        resource = TestResource()
+        obj = TestObject()
+        data = resource.serialize_object(obj, request=request)
+
+        self.assertEqual(data, {
+            '_expanded': {
+                'field2': {
+                    'item_mimetype': 'application/vnd.djblets-test.user+json',
+                },
+            },
+            'field2': {
+                'links': {
+                    'self': {
+                        'href': 'http://testserver/api/test/users/1/',
+                        'method': 'GET',
+                    },
+                },
+            },
+            'links': {
+                'self': {
+                    'href': 'http://testserver/api/test/?only-fields=field2'
                             '&expand=field2',
                     'method': 'GET',
                 },
                 'field3': {
-                    'href': 'http://testserver/api/test1/',
+                    'href': 'http://testserver/api/test/users/2/',
                     'method': 'GET',
-                    'title': 'Test',
+                    'title': 'user2',
                 },
             }
         })
@@ -465,13 +736,13 @@ class WebAPIResourceTests(TestCase):
             allowed_methods = ('GET', 'POST', 'PUT', 'DELETE')
             fields = {
                 'field1': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
                 'field2': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
                 'field3': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
             }
 
@@ -506,13 +777,13 @@ class WebAPIResourceTests(TestCase):
             allowed_methods = ('GET', 'POST', 'PUT', 'DELETE')
             fields = {
                 'field1': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
                 'field2': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
                 'field3': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 },
             }
 
@@ -539,7 +810,7 @@ class WebAPIResourceTests(TestCase):
         resource = WebAPIResource()
         resource.fields = {
             'my_field': {
-                'type': six.text_type,
+                'type': StringFieldType,
             }
         }
 
@@ -599,13 +870,13 @@ class WebAPIResourceTests(TestCase):
                 self.name = name
                 self.pk = pk  # Django 1.8+ requires a pk field
 
-        class TestResource(WebAPIResource):
+        class TestResource(BaseTestWebAPIResource):
             fields = {
                 'dependency': {
                     'type': [TestObject],
                 },
                 'name': {
-                    'type': six.text_type,
+                    'type': StringFieldType,
                 }
             }
 
@@ -623,6 +894,12 @@ class WebAPIResourceTests(TestCase):
 
             self.maxDiff = 100000
             self.assertEqual(data, {
+                '_expanded': {
+                    'dependency': {
+                        'item_mimetype':
+                            'application/vnd.djblets-test.testresource+json',
+                    },
+                },
                 'dependency': {
                     'links': {
                         'dependency': {
